/**  Shapes.cpp

**/

#include "myheaders.h"
#include "materials.h"



materialStruct GreyFlat = {
  {0.2, 0.2, 0.2, 1.0},
  {0.5, 0.5, 0.5, 1.0},
  {0.1, 0.1, 0.1, 1.0},
  {0.0}
};

materialStruct Sky = {
  {.1, .1, .1, .1},
  {.5, .5, .7, .1},
  {.1, .2, .2, .1},
  {0.0}
};
materialStruct PeachFlat = {
  {0.0, 0.0, 0.0, 1.0},
  {0.5, 0.4, 0.3, 1.0},
  {0.1, 0.1, 0.1, 1.0},
  {0.0}
};

materialStruct RedFlat = {
  {0.3, 0.0, 0.0, 1.0},
  {0.9, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};

materialStruct GreenShiny = {
  {0.0, 0.1, 0.0, 1.0},
  {0.0, 0.2, 0.0, 1.0},
  {0.2, 0.2, 0.2, 1.0},
  {0.0}
};

materialStruct Sun= {
  {1, 1, 1, 1},
  {1, .9, .5, 1},
  {.5, .5, .4, 1},
  {128.0}
};
materialStruct Default = {
  {0.2, 0.2, 0.2, 1.0},
  {0.8, 0.8, 0.8, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {8.0}
};

materialStruct WhiteShiny = {
  {1.0, 1.0, 1.0, 1.0},
  {1.0, 1.0, 1.0, 1.0},
  {1.0, 1.0, 1.0, 1.0},
  {100.0}
};

materialStruct Brass = {
  {0.33, 0.22, 0.03, 1.0},
  {0.78, 0.57, 0.11, 1.0},
  {0.99, 0.91, 0.81, 1.0},
  {27.8}
};

materialStruct Cloud = {
  {.6, .6, .6, 1.0},
  {1, 1, 1, 1.0},
  {0, 0, 0, 1.0},
  {0}
};

GLfloat emissive[4] = {1.0, 1.0, 1.0, 1.0};
GLfloat blank[4] = {0, 0, 0, 0};

// draws the sun
void DrawSun()
{
	materials(Sun);
	GLUquadricObj *sun = gluNewQuadric();
	glColor3ub(255, 215, 0);
	glEnable(GL_COLOR_MATERIAL);
	glDisable(GL_LIGHTING);
	glPushMatrix();
		//glTranslatef(0, 40, 20);
	
		gluSphere(sun, 1, 50, 50);
	glPopMatrix();
	glEnable(GL_LIGHTING);
	glDisable(GL_COLOR_MATERIAL);
}

// renders a street light model
void DrawStreetLight(float y)
{
	GLUquadricObj * pole = gluNewQuadric();

	glPushMatrix();
		glTranslatef(0, .9, 0);
		glScalef(.1, .25, .1);
	glPushMatrix();
		materials(WhiteShiny);
		if(y <  0)
		{
			glMaterialfv(GL_FRONT, GL_EMISSION, emissive);
		}
		glColor3f(.9, .9, .9);
		glTranslatef(0, 0, 0);
		glutSolidSphere(.2, 20, 20);
		glMaterialfv(GL_FRONT, GL_EMISSION, blank);
	glPopMatrix();

	glPushMatrix();

		glRotatef(90, 1, 0, 0);
		glColor3f(0, 0, 0);
		materials(Brass);
		gluCylinder(pole, .1, .1, 2, 10, 10);
	glPopMatrix();
	glPopMatrix();
}



// draw sidewalk and grass terrain
void DrawTerrain()
{
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);
	glBindTexture(GL_TEXTURE_2D, 4);
	glColor3ub(102, 204, 51);
	materials(GreenShiny);
	glPushMatrix();
		glTranslatef(-25, .4, 25);
		glScalef(42.5, 1, 42.5);
		glBegin(GL_POLYGON);
			glTexCoord2f(0, 0);
			glVertex3f(-.5, 0, -.5);
			glTexCoord2f(1, 0);
			glVertex3f(-.5, 0, .5);
			glTexCoord2f(1, 1);
			glVertex3f(.5, 0, .5);
			glTexCoord2f(0, 1);
			glVertex3f(.5, 0, -.5);
		glEnd();
	glPopMatrix();
	materials(GreyFlat);
	glBindTexture(GL_TEXTURE_2D, 2);
	glColor3f(.5, .5, .5);
	glPushMatrix();
		glTranslatef(25, .25, -25);
		glScalef(45, .25, 45);
		glutSolidCube(1);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(25, .25, 25);
		glScalef(45, .25, 45);
		glutSolidCube(1);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-25, .25, -25);
		glScalef(45, .25, 45);
		glutSolidCube(1);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(-25, .25, 25);
		glScalef(45, .25, 45);
		glutSolidCube(1);
	glPopMatrix();
	
	glDisable(GL_TEXTURE_2D);
	DrawStreetPaint();
	glDisable(GL_COLOR_MATERIAL);
}

// draw street paint lines
void DrawStreetPaint()
{
	glPushMatrix();
	glColor3ub(255, 255, 0);
	glLineWidth(2.0);
	glBegin(GL_LINES);
	for(int i = -50; i < 50; i+=2)
	{
		glVertex3f(i, .01, 0);
		glVertex3f(i+1, .01, 0);
	}
	for(int i = -50; i < 50; i+=2)
	{
		glVertex3f(0, .01, i);
		glVertex3f(0, .01, i+1);
	}
	glEnd();
	glPopMatrix();
}

//skyscraper model
void DrawSkyscraper()
{
	GLUquadricObj * skyscraper_roof = gluNewQuadric();
	gluQuadricTexture(skyscraper_roof, GL_TRUE);
	glColor3f(.5, .5, .5);
	glPushMatrix();
	glTranslatef(0, 1, 0);
	glScalef(1, 1.25, 1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 5);
	materials(GreyFlat);
	glPushMatrix();
		glTranslatef(0, 13, 0);
		glRotatef(-90, 1, 0, 0);
		gluCylinder(skyscraper_roof,.5, 0, 6, 4, 10);
	glPopMatrix();

	glPushMatrix();
		glTranslatef(0, 11.5, 0);
		DrawCube(3);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, 9.5, 0);
		glScalef(1, .5, 1);
		DrawCube(6);
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0, 4.5, 0);
		DrawCube(7);
	glPopMatrix();
	glPushMatrix();
		
		glScalef(1, .5, 1);
		DrawCube(8);
	glPopMatrix();
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
}

//skyscraper model 1
void DrawSkyscraper1()
{
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	GLUquadricObj * skyscraper_roof = gluNewQuadric();
	gluQuadricTexture(skyscraper_roof, GL_TRUE);
	glColor3f(.5, .5, .5);
	materials(GreyFlat);
	glBindTexture(GL_TEXTURE_2D, 7);
	glPushMatrix();
		glTranslatef(0, 3, 0);
		glScalef(1, 2.5, 1);
	glPushMatrix();
		glTranslatef(0, 3.5, 0);
		//glScalef(.9, 1, .9);
		glRotatef(-45, 0, 1, 0);
		glRotatef(-90, 1, 0, 0);
		gluCylinder(skyscraper_roof, 3, 0, 1, 4, 1);	
	glPopMatrix();
	glBindTexture(GL_TEXTURE_2D, 6);
	glPushMatrix();
		glTranslatef(0, 3.3, 0);
		glScalef(.9, .1, .9);
		DrawCube(5);
	glPopMatrix();

	glPushMatrix();
		glScalef(1, 1.25, 1);
		DrawCube(5);
	glPopMatrix();
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
}

// flat top skyscraper model 2
void DrawSkyscraper2()
{
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	materials(GreyFlat);
	glColor3f(.5, .5, .5);
	glBindTexture(GL_TEXTURE_2D, 8);
	glPushMatrix();
		glTranslatef(0, 5, 0);
		glScalef(1, 2, 1);
		DrawCube(5);
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
}


// wide rectangular building
void DrawWideBuilding()
{
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	materials(GreyFlat);
	glColor3f(.5, .5, .5);
	glBindTexture(GL_TEXTURE_2D, 9);
	glPushMatrix();
		glTranslatef(0, 2.5, 0);
		glScalef(2, 1, 1);
		DrawCube(5);
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
}


void DrawWideBuilding1()
{
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	materials(GreyFlat);
	glColor3f(.5, .5, .5);
	glBindTexture(GL_TEXTURE_2D, 10);
	glPushMatrix();
		glTranslatef(0, 5, 0);
		glScalef(2, 2, 1);
		DrawCube(5);
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
}


void DrawBuilding2()
{
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	materials(GreyFlat);
	glColor3f(.5, .5, .5);
	glBindTexture(GL_TEXTURE_2D, 11);
	glPushMatrix();
		glScalef(3, 3, 3);
		glTranslatef(0, 1.15, 0);
		glPushMatrix();
			glTranslatef(.75, -.5, .6);
			glScalef(1, 10, 1);
			DrawCube(.1);
		glPopMatrix();

		glPushMatrix();
			glTranslatef(-.75, -.5, .6);
			glScalef(1, 10, 1);
			DrawCube(.1);
		glPopMatrix();

		//base
		glPushMatrix();
			glTranslatef(0, -1, 0);
			glScalef(2, .25, 1.5);
			DrawCube(1);
		glPopMatrix();

		//body
		glPushMatrix();
			glTranslatef(0, -.5, -.2);
			glScalef(1.75, 1, 1);
			DrawCube(1);
		glPopMatrix();
	
		//roof
		glPushMatrix();
			glTranslatef(0, .25, 0);
			glScalef(1.75, .25, 1.25);
			DrawCube(1);
		glPopMatrix();

		//roof
		glPushMatrix();
			glScalef(2, .25, 1.5);
			DrawCube(1);
		glPopMatrix();

	glPopMatrix();
}

void DrawBuilding1()
{
	materials(PeachFlat);
	glColor3ub(255,218,185);
	glPushMatrix();
		glScalef(3,3,3);
		glTranslatef(0, -.42, 0);
		glPushMatrix();
			glScalef(2, .5, 1.5);
			glutSolidCube(1);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(.25, .5, 0);
			glScalef(1.5, .5, 1.5);
			glutSolidCube(1);
		glPopMatrix();
	glPopMatrix();
}


// draws world floor plane
void DrawFloor()
{
	//glOrtho( -2.0*(float)w/h, 2.0*(float)w/h, -2.0, 2.0, 1.0, 15.0);
	//glDisable(GL_LIGHTING);
	materials(GreyFlat);
	glColor3f(.5, .5, .5);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 2);
	for(int i = -50; i < 50; i++)
	{
		for(int j = -50; j < 50; j++)
		{
			glBegin(GL_POLYGON);
			glTexCoord2f(0, 0);
			glVertex3f(i, 0, j);
			glTexCoord2f(1, 0);
			glVertex3f(i, 0, j+1);
			glTexCoord2f(1, 1);
			glVertex3f(i+1, 0, j+1);
			glTexCoord2f(0, 1);
			glVertex3f(i+1, 0, j);
			glEnd();
		}
	}
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_TEXTURE_2D);
	//glEnable(GL_LIGHTING);


}

//tree model
void DrawTree()
{
	GLUquadricObj * tree_trunk = gluNewQuadric();
	//glDisable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	glPushMatrix();
	glTranslatef(0, .4, 0);
	// setting up tree leaves
	glPushMatrix();
		glRotatef(90, 0, 1, 0);
		glTranslatef(0, .8, 0);
		DrawTreeLeaves(.025, 0.01, .25, 20);
	glPopMatrix();

	glPushMatrix();
		glRotatef(60, 0, 1, 0);
		glTranslatef(0, .7, 0);
		DrawTreeLeaves(.05, .0375, .25, 20);
	glPopMatrix();
	glPushMatrix();
		glRotatef(30, 0, 1, 0);
		glTranslatef(0, .6, 0);
		DrawTreeLeaves(.1, .075, .25, 20);
	glPopMatrix();

	glPushMatrix();
		glRotatef(15, 0, 1, 0);
		glTranslatef(0, .5, 0);
		DrawTreeLeaves(.2, .1, .25, 20);
	glPopMatrix();

	glPushMatrix();
		glTranslatef(0, .4, 0);
		DrawTreeLeaves(.2, .1, .25, 20);
	glPopMatrix();

	//trunk
	glColor3ub(139, 69, 19);
	materials(Brass);
	glPushMatrix();
		glRotatef(-90, 1, 0, 0);
		gluCylinder(tree_trunk, .06, .025, .4, 5, 1);
	glPopMatrix();
	glPopMatrix();
	glDisable(GL_COLOR_MATERIAL);
	//glEnable(GL_LIGHTING);
}

// draws tree leaves given params
void DrawTreeLeaves(float base, float top, float height, int numLeaves)
{
	glEnable(GL_COLOR_MATERIAL);
	materials(GreenShiny);

	glPushMatrix();
		glColor3ub(34, 139, 34);
		glBegin(GL_TRIANGLES);
		for(float i = 0; i < 2*PI*numLeaves; i += 2*PI/numLeaves)
		{
			glVertex3f(top*cos(i), 0, top*sin(i));
			glVertex3f(top*cos(i) + cos(i) *base, -height, top*sin(i) + sin(i)*base);
			glVertex3f(top*cos(i + 2*PI/numLeaves), 0, top*sin(i+2*PI/numLeaves));
		}
		glEnd();
	glPopMatrix();

	glDisable(GL_COLOR_MATERIAL);
}

// draw 6 sided cube @ origin of current matrix using triangle strips
// texture predetermined by calling function
void DrawCube(float length)
{
	glBegin(GL_POLYGON);
		glTexCoord2f(0, 0);
		glVertex3f(-length/2, length/2, -length/2); //back  top left
		glTexCoord2f(0, 1);
		glVertex3f(length/2, length/2, -length/2); //back top right
		glTexCoord2f(1, 1);
		glVertex3f(length/2, length/2, length/2); //front top right;
		glTexCoord2f(1, 0);
		glVertex3f(-length/2, length/2, length/2); //front top left

	glEnd();

	glBegin(GL_POLYGON);
		glTexCoord2f(0, 0);
		glVertex3f(-length/2, -length/2, -length/2); //back bottom left
		glTexCoord2f(0, 1);
		glVertex3f(length/2, -length/2, -length/2); //back bottom right
		glTexCoord2f(1, 1);
		glVertex3f(length/2, -length/2, length/2); // front bottom right;
		glTexCoord2f(1, 0);
		glVertex3f(-length/2, -length/2, length/2); //front bottom left
	glEnd();

		
	
	glBegin(GL_POLYGON); // front
		glTexCoord2f(0, 0);
		glVertex3f(-length/2, length/2, length/2); //front top left
		glTexCoord2f(1, 0);
		glVertex3f(-length/2, -length/2, length/2); //front bottom left
		glTexCoord2f(1, 1);
		glVertex3f(length/2, -length/2, length/2); // front bottom right;
		glTexCoord2f(0, 1);
		glVertex3f(length/2, length/2, length/2); //front top right;
	glEnd();

	glBegin(GL_POLYGON); //left 
		glTexCoord2f(0, 0);
		glVertex3f(-length/2, length/2, -length/2); //back  top left
		glTexCoord2f(1, 0);
		glVertex3f(-length/2, -length/2, -length/2); //back bottom left

		glTexCoord2f(1, 1);
		glVertex3f(-length/2, -length/2, length/2); //front bottom left
		glTexCoord2f(0, 1);
		glVertex3f(-length/2, length/2, length/2); //front top left
	glEnd();

	glBegin(GL_POLYGON); //back
		glTexCoord2f(0, 0);
		glVertex3f(-length/2, length/2, -length/2); //back  top left
		glTexCoord2f(1, 0);
		glVertex3f(-length/2, -length/2, -length/2); //back bottom left
		glTexCoord2f(1, 1);
		glVertex3f(length/2, -length/2, -length/2); //back bottom right
		glTexCoord2f(0, 1);
		glVertex3f(length/2, length/2, -length/2); //back top right
	glEnd();

	glBegin(GL_POLYGON); //right
		glTexCoord2f(0, 0);
		glVertex3f(length/2, length/2, length/2); //front top right;
		glTexCoord2f(1, 0);
		glVertex3f(length/2, -length/2, length/2); // front bottom right;
		glTexCoord2f(1, 1);
		glVertex3f(length/2, -length/2, -length/2); //back bottom right
		glTexCoord2f(0, 1);
		glVertex3f(length/2, length/2, -length/2); //back top right
	glEnd();
	
}















