/** skydome.cpp
    Generates a simple skydome
**/

#include "myheaders.h"

using namespace std;

GLfloat night_color[3] = {.05882f,.05882f,0.43922f};
GLfloat day_color[3] = {0.52941f,0.80784f, 0.92157f};
GLfloat cur_color[3] = {.05882f,.05882f,0.43922f};

int thetaSky, phiSky;
float radius = 50.0;

int dtheta = 15;
int dphi = 15;

int NumVertices = (int)((360/dtheta)*(90/dphi)*4);


Vector3 * vertices = (Vector3*)malloc(sizeof(Vector3) * NumVertices);
Vector3 stars[50];

void CalcSkyDomeVertices()
{
	int n = 0;
	float magnitude;

	for (phiSky=0; phiSky <= 90 - dphi; phiSky += (int)dphi) 
	{
		for (thetaSky=0; thetaSky <= 360 - dtheta; thetaSky += (int)dtheta) 
		{
			
			vertices[n].x = radius * sin(phiSky*DTOR) * cos(DTOR*thetaSky);
			vertices[n].y = radius * sin(phiSky*DTOR) * sin(DTOR*thetaSky);
			vertices[n].z = radius * cos(phiSky*DTOR);
			magnitude = sqrt(pow(vertices[n].x ,2) + pow(vertices[n].y, 2) + pow(vertices[n].z, 2));

			//vertices[n].u = (vertices[n].x)/(radius*2);
			//vertices[n].v = (vertices[n].y)/(radius*2);
			vertices[n].u = (float)(atan2(vertices[n].x/magnitude, vertices[n].z/magnitude)/(PI*2)) + 0.5f;
			vertices[n].v = (float)(asin(vertices[n].y/magnitude) / PI) + 0.5f;

			n++;


			vertices[n].x = radius * sin((phiSky+dphi)*DTOR) * cos(thetaSky*DTOR);
			vertices[n].y = radius * sin((phiSky+dphi)*DTOR) * sin(thetaSky*DTOR);
			vertices[n].z = radius * cos((phiSky+dphi)*DTOR);
			magnitude = sqrt(pow(vertices[n].x ,2) + pow(vertices[n].y, 2) + pow(vertices[n].z, 2));

			//vertices[n].u = (vertices[n].x)/(radius*2);
			//vertices[n].v = (vertices[n].y)/(radius*2);
			vertices[n].u = (float)(atan2(vertices[n].x/magnitude, vertices[n].z/magnitude)/(PI*2)) + 0.5f;
			vertices[n].v = (float)(asin(vertices[n].y/magnitude) / PI) + 0.5f;
			n++;


			vertices[n].x = radius * sin(DTOR*phiSky) * cos(DTOR*(thetaSky+dtheta));
			vertices[n].y = radius * sin(DTOR*phiSky) * sin(DTOR*(thetaSky+dtheta));
			vertices[n].z = radius * cos(DTOR*phiSky);
			magnitude = sqrt(pow(vertices[n].x ,2) + pow(vertices[n].y, 2) + pow(vertices[n].z, 2));

			//vertices[n].u = (vertices[n].x)/(radius*2);
			//vertices[n].v = (vertices[n].y)/(radius*2);
			vertices[n].u = (float)(atan2(vertices[n].x/magnitude, vertices[n].z/magnitude)/(PI*2)) + 0.5f;
			vertices[n].v = (float)(asin(vertices[n].y/magnitude) / PI) + 0.5f;
			n++;


			if (phiSky > -90 && phiSky < 90) 
			{

				vertices[n].x = radius * sin((phiSky+dphi)*DTOR) * cos(DTOR*(thetaSky+dtheta));
				vertices[n].y = radius * sin((phiSky+dphi)*DTOR) * sin(DTOR*(thetaSky+dtheta));
				vertices[n].z = radius * cos((phiSky+dphi)*DTOR);
				magnitude = sqrt(pow(vertices[n].x ,2) + pow(vertices[n].y, 2) + pow(vertices[n].z, 2));

				//vertices[n].u = (vertices[n].x)/(radius*2);
				//vertices[n].v = (vertices[n].y)/(radius*2);

				vertices[n].u = (float)(atan2(vertices[n].x/magnitude, vertices[n].z/magnitude)/(PI*2)) + 0.5f;
				vertices[n].v = (float)(asin(vertices[n].y/magnitude) / PI) + 0.5f;
				n++;
			}
		}
	}
	MapSkyTexCoords();
	//sky dome ready to be drawn
}


// maps texture coordinates for each vertices
void MapSkyTexCoords()
{
	for (int i=0; i < NumVertices-2; i++)
	{
        if (vertices[i].u - vertices[i+1].u > 0.9f)
                vertices[i+1].u += 1.0f;
        if (vertices[i+1].u - vertices[i].u > 0.9f)
                vertices[i].u += 1.0f;
        if (vertices[i].u - vertices[i+2].u > 0.9f)
                vertices[i+2].u += 1.0f;
        if (vertices[i+2].u - vertices[i].u > 0.9f)
                vertices[i].u += 1.0f;
        if (vertices[i+1].u - vertices[i+2].u > 0.9f)
                vertices[i+2].u += 1.0f;
        if (vertices[i+2].u - vertices[i+1].u > 0.9f)
                vertices[i+1].u += 1.0f;
        if (vertices[i].v - vertices[i+1].v > 0.8f)
                vertices[i+1].v += 1.0f;
        if (vertices[i+1].v - vertices[i].v > 0.8f)
                vertices[i].v += 1.0f;
        if (vertices[i].v - vertices[i+2].v > 0.8f)
                vertices[i+2].v += 1.0f;
        if (vertices[i+2].v - vertices[i].v > 0.8f)
                vertices[i].v += 1.0f;
        if (vertices[i+1].v - vertices[i+2].v > 0.8f)
                vertices[i+2].v += 1.0f;
        if (vertices[i+2].v - vertices[i+1].v > 0.8f)
                vertices[i+1].v += 1.0f;
	}

}

// debug to print all verticies
void printSkyVertices()
{
	for(int i = 0; i < NumVertices; i++)
	{
		printf("Vertex %d:  x %f y %f z %f u %f v %f\n", i, vertices[i].x, vertices[i].y, vertices[i].z, vertices[i].u, vertices[i].v);
	}

}

void CalcStars()
{
	Vector3 randVertex;
	for(int i = 0; i < 50; i++)
	{
		randVertex = vertices[rand()%NumVertices];
		stars[i].x = randVertex.x;
		stars[i].y = randVertex.y;
		stars[i].z = randVertex.z;
	}
}


void DrawStars()
{
	glPointSize(3.0f);
	glColor3ub(255, 215, 0);
	glDisable(GL_LIGHTING);
	glBegin(GL_POINTS);
	for(int i = 0; i < 50; i++)
	{
		glVertex3f(stars[i].x, stars[i].y, stars[i].z);
	}
	glEnd();
	glEnable(GL_LIGHTING);
}

void DrawSkyDomeMesh()
{
	glDisable(GL_LIGHTING);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	glFrontFace(GL_CW);
	//glBindTexture(GL_TEXTURE_2D, 1);
	glPushMatrix();
	glTranslatef(0, 0, 0);
	glRotatef(-90, 1, 0, 0);
	glColor3f(0, 0, 0);

	//wireframe
	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	//glBegin(GL_TRIANGLE_STRIP);
	//	for(int i = 0; i < NumVertices; i++)
	//	{
	//		glBindTexture(GL_TEXTURE_2D, 1);
	//		glTexCoord2f(vertices[i].u, vertices[i].v);
	//		glVertex3f(vertices[i].x, vertices[i].y, vertices[i].z);
	//	}
	//glEnd();

	//filled
	glColor3fv(cur_color);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glBindTexture(GL_TEXTURE_2D, 3);
	glEnable(GL_TEXTURE_2D);
	glBegin(GL_TRIANGLE_STRIP);
		for(int i = 0; i < NumVertices; i++)
		{

			glTexCoord2f(vertices[i].u, vertices[i].v);
			glVertex3f(vertices[i].x, vertices[i].y, vertices[i].z);
		}
	glEnd();
	glPopMatrix();
	glFrontFace(GL_CCW);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
}

void adjust_sky(float x, float y)
{
	float rate = .0005f;
	if(x > 0 && y > 0)
	{
		//printf("shifting to day\n");
		if((y < 10))
		{
			rate = .0025f;
		}
		if(cur_color[0] < day_color[0])
		{
			cur_color[0]+= rate;
		}
		if(cur_color[1] < day_color[1])
		{
			cur_color[1]+= rate;
		}
		if(cur_color[2] < day_color[2])
		{
			cur_color[2]+= rate;
		}
		
	}
	else if((x < 0 && y > 0))
	{
		if(y < 25)
		{
		if(y<10)
		{
			rate = .002f;
		}
		if(cur_color[0] > night_color[0])
		{
			cur_color[0]-= rate;
		}
		if(cur_color[1] > night_color[1])
		{
			cur_color[1]-= rate;
		}
		if(cur_color[2] > night_color[2])
		{
			cur_color[2]-= rate;
		}
		}
	}
	else
	{
		cur_color[0] = night_color[0];
		cur_color[1] = night_color[1];
		cur_color[2] = night_color[2];
	}
	//printf("x %f y %f %f %f %f \n", x, y, cur_color[0], cur_color[1], cur_color[2]);
}
